﻿using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Linq;
using System.Text;
using System.Windows.Forms;
using System.Numerics;

namespace WindowsFormsApplication1 {
   public partial class Form1 : Form {
      public Form1() {
         InitializeComponent();
      }

      private void tColorA_Scroll(object sender, EventArgs e) {
         ApplyChange();
         SetHSB();
         SetLabelsHSB();
      }

      private void tColorR_Scroll(object sender, EventArgs e) {
         ApplyChange();
         SetHSB();
         SetLabelsHSB();
      }

      private void tColorG_Scroll(object sender, EventArgs e) {
         ApplyChange();
         SetHSB();
         SetLabelsHSB();
      }

      private void tColorB_Scroll(object sender, EventArgs e) {
         ApplyChange();
         SetHSB();
         SetLabelsHSB();
      }

      private void tColorBright_Scroll(object sender, EventArgs e) {
         labelBright.Text = ((float)tColorBright.Value/100).ToString("F");
         SetRGB();
         ApplyChange();
      }

      private void tColorHue_Scroll(object sender, EventArgs e) {
         labelHue.Text = tColorHue.Value.ToString();
         SetRGB();
         ApplyChange();
      }

      private void tColorSatur_Scroll(object sender, EventArgs e) {
         labelSatur.Text = ((float)tColorSatur.Value/100).ToString("F");
         SetRGB();
         ApplyChange();
      }


      private void ApplyChange() {
         FillPanel();
         SetLabels();
         SetHex();
         
         SetWindowText();
      }

      private void SetLabels() {
         labelA.Text = tColorA.Value.ToString();
         labelR.ForeColor = Color.FromArgb(tColorR.Value, 0, 0);
         labelR.Text = tColorR.Value.ToString("X2");
         labelG.ForeColor = Color.FromArgb(0, tColorG.Value, 0);
         labelG.Text = tColorG.Value.ToString("X2");
         labelB.ForeColor = Color.FromArgb(0, 0, tColorB.Value);
         labelB.Text = tColorB.Value.ToString("X2");
      }

      private void SetLabelsHSB() {
         labelBright.Text = Panel.BackColor.GetBrightness().ToString(); 
         labelHue.Text = Panel.BackColor.GetHue().ToString(); 
         labelSatur.Text = Panel.BackColor.GetSaturation().ToString(); 
      }

      private void SetHex() {
         string hex;
         hex = "#" + labelR.Text + labelG.Text + labelB.Text;
         labelH.Text = hex;
         Clipboard.SetText(hex);
      }

      private void SetWindowText() {
         this.Text = "Alpha:" + tColorA.Value + " Red:" + tColorR.Value
            + " Green: " + tColorG.Value
            + " Blue: " + tColorB.Value + " ";
      }

      private void FillPanel() {
         Panel.BackColor = Color.FromArgb(tColorA.Value, tColorR.Value, tColorG.Value, tColorB.Value);
      }

      private void Form1_FormClosed(object sender, FormClosedEventArgs e) {

         SaveProperties();

         for (; Opacity > 0; Opacity -= 0.01) {
            System.Threading.Thread.Sleep(1);
         }
      }

      private void Form1_FormClosing(object sender, FormClosingEventArgs e) {

         bool changedValues = false;
         if (Properties.Settings.Default.R != (byte)tColorR.Value
            || Properties.Settings.Default.G != (byte)tColorG.Value
            || Properties.Settings.Default.B != (byte)tColorB.Value) {
            changedValues = true;
         }

         if (changedValues) {
            DialogResult dr = MessageBox.Show("Zamknac?", this.Text, MessageBoxButtons.YesNo);
            if (dr == System.Windows.Forms.DialogResult.No) {
               e.Cancel = true;
            }
         }
      }

      private void btnReset_Click(object sender, EventArgs e) {
         tColorA.Value = 255;
         tColorR.Value = 0;
         tColorG.Value = 0;
         tColorB.Value = 0;
         ApplyChange();
         SetHSB();
         SetLabelsHSB();
         cbColor.Text = string.Empty;
      }

      private void btnExit_Click(object sender, EventArgs e) {
         this.Close();
      }

      private void Form1_Load(object sender, EventArgs e) {

         LoadProperties();

         foreach (string colorName in Enum.GetNames(typeof(KnownColor))) {
            cbColor.Items.Add(colorName);
         }
         ApplyChange();
         SetHSB();
         SetLabelsHSB();
      }

      private void cbColor_SelectedValueChanged(object sender, EventArgs e) {
         Color selectedColor = Color.FromName(cbColor.SelectedItem.ToString());
         tColorA.Value = selectedColor.A;
         tColorR.Value = selectedColor.R;
         tColorG.Value = selectedColor.G;
         tColorB.Value = selectedColor.B;
         ApplyChange();
         SetHSB();
         SetLabelsHSB();
      }

      private void SaveProperties() {

         Properties.Settings.Default.R = (byte)tColorR.Value;
         Properties.Settings.Default.G = (byte)tColorG.Value;
         Properties.Settings.Default.B = (byte)tColorB.Value;
         Properties.Settings.Default.Left = this.Left;
         Properties.Settings.Default.Top = this.Top;
         Properties.Settings.Default.Width = this.Width;
         Properties.Settings.Default.Height = this.Height;

         Properties.Settings.Default.Save();
      }

      private void HSBToRGB(float h, float s, float br, out int r, out int g, out int b) {

         int ibr = (int)(br * 255);
         if (s == 0.0f) {
            r = g = b = ibr;
            return;
         }
         float fsector = h / 60;
         int sector = (int)Math.Floor(fsector);
         float factor = fsector - sector;

         int s1 = (int)(ibr * (1 - s));
         int s2 = (int)(ibr * (1 - s * factor));
         int s3 = (int)(ibr * (1 - s * (1 - factor)));

         switch (sector) {
            case 0:
               r = ibr; g = s3; b = s1; break;
            case 1:
               r = s2; g = ibr; b = s1; break;
            case 2:
               r = s1; g = ibr; b = s3; break;
            case 3:
               r = s1; g = s2; b = ibr; break;
            case 4:
               r = s3; g = s1; b = ibr; break;
            default:
               r = ibr; g = s1; b = s2; break;
         }

      }

      private void SetRGB() {
         int r, g, b;
         HSBToRGB((float)(tColorHue.Value), (float)(tColorSatur.Value)/100, (float)(tColorBright.Value) / 100, out r, out  g, out b);
         tColorA.Value = 255;
         tColorR.Value = r;
         tColorG.Value = g;
         tColorB.Value = b;
      }


      private void SetHSB() {
         tColorBright.Value = (int)(Panel.BackColor.GetBrightness() * 100);
         tColorHue.Value = (int)Panel.BackColor.GetHue();
         tColorSatur.Value = (int)(Panel.BackColor.GetSaturation() * 100);
      }

      private void LoadProperties() {

         tColorA.Value = 255;
         tColorR.Value = Properties.Settings.Default.R;
         tColorG.Value = Properties.Settings.Default.G;
         tColorB.Value = Properties.Settings.Default.B;

         this.Left = Properties.Settings.Default.Left;
         this.Top = Properties.Settings.Default.Top;
         this.Width = Properties.Settings.Default.Width;
         this.Height = Properties.Settings.Default.Height;
      }
   }
}